#include <windows.h>
#include <math.h>
#include <stdlib.h>

#define BTN_CLEAR 1000

char szButton[] = "BUTTON";
char szEmpty[] = "";
char szClassName[] = "PegSolitaire";
char szAppTitle[] = "Peg Solitaire";

int board[7][7];
HWND pegs[7][7];

int coords_length;
int x_coord[4];
int y_coord[4];

int InCross(int x, int y) { return x >= 2 && x <= 4 || y >= 2 && y <= 4; }

void InitBoard() {
  int x, y;
  for (x = 0; x < 7; ++x) {
    for (y = 0; y < 7; ++y) {
      board[x][y] = InCross(x, y) && !(3 == x && 3 == y);
    }
  }
  coords_length = 0;
}

void RedrawBoard() {
  int x, y;
  for (x = 0; x < 7; ++x) {
    for (y = 0; y < 7; ++y) {
      if (InCross(x, y)) {
        SendMessage(pegs[x][y], WM_SETTEXT, 0,
                    (LPARAM)(board[x][y] ? "o" : szEmpty));
      }
    }
  }
}

void Renew() {
  InitBoard();
  RedrawBoard();
}

int sign(int x) { return x > 0 ? 1 : (0 == x ? 0 : -1); }

int IsOnOneLine(int a1, int a2, int a3, int b1, int b2, int b3) {
  return (b1 == b2) && (b1 == b3) && abs(a1 - a2) == 1 && abs(a1 - a3) == 2 &&
         abs(a2 - a3) == 1 && sign(a1 - a2) == sign(a2 - a3) &&
         sign(a1 - a2) == sign(a1 - a3);
}

void MaybeWin() {
  int count = 0, x, y;
  for (x = 0; x < 7; ++x) {
    for (y = 0; y < 7; ++y) {
      if (board[x][y]) {
        ++count;
      }
    }
  }
  if (1 == count) {
    Renew();
  }
}

void TryMakeMove() {
  if ((IsOnOneLine(x_coord[0], x_coord[1], x_coord[2], y_coord[0], y_coord[1],
                   y_coord[2]) ||
       IsOnOneLine(y_coord[0], y_coord[1], y_coord[2], x_coord[0], x_coord[1],
                   x_coord[2])) &&
      board[x_coord[0]][y_coord[0]] && board[x_coord[1]][y_coord[1]] &&
      !board[x_coord[2]][y_coord[2]]) {
    board[x_coord[0]][y_coord[0]] = 0;
    board[x_coord[1]][y_coord[1]] = 0;
    board[x_coord[2]][y_coord[2]] = 1;
    coords_length = 0;
    RedrawBoard();
    MaybeWin();
  }
}

void BoardClicked(int x, int y) {
  x_coord[coords_length] = x;
  y_coord[coords_length] = y;
  ++coords_length;
  if (coords_length > 3) {
    x_coord[0] = x_coord[1];
    x_coord[1] = x_coord[2];
    x_coord[2] = x_coord[3];
    y_coord[0] = y_coord[1];
    y_coord[1] = y_coord[2];
    y_coord[2] = y_coord[3];
    --coords_length;
  }
  if (3 == coords_length) {
    TryMakeMove();
  }
}

LRESULT CALLBACK
WindowProcedure(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam) {
  int x, y, idButton;

  switch (message) {
  case WM_DESTROY:
    PostQuitMessage(0);
    break;
  case WM_COMMAND:
    if (BN_CLICKED == HIWORD(wParam)) {
      idButton = LOWORD(wParam);
      if (BTN_CLEAR == idButton) {
        Renew();
      } else {
        x = idButton / 10;
        y = idButton % 10;
        BoardClicked(x, y);
      }
    }
    break;
  default:
    return DefWindowProc(hwnd, message, wParam, lParam);
  }

  return 0;
}

int WINAPI WinMain(HINSTANCE hThisInstance, HINSTANCE hPrevInstance,
                   LPSTR lpszArgument, int nCmdLine) {
  HWND hwnd;
  MSG messages;
  WNDCLASSEX wincl;
  HWND hwndClear;

  int x, y;

  wincl.hInstance = hThisInstance;
  wincl.lpszClassName = szClassName;
  wincl.lpfnWndProc = WindowProcedure;
  wincl.style = CS_DBLCLKS;
  wincl.cbSize = sizeof(WNDCLASSEX);
  wincl.hIcon = LoadIcon(NULL, IDI_APPLICATION);
  wincl.hIconSm = LoadIcon(NULL, IDI_APPLICATION);
  wincl.hCursor = LoadCursor(NULL, IDC_ARROW);
  wincl.lpszMenuName = NULL;
  wincl.cbClsExtra = 0;
  wincl.cbWndExtra = 0;
  wincl.hbrBackground = (HBRUSH)COLOR_BACKGROUND;

  if (!RegisterClassEx(&wincl))
    return 0;

  hwnd = CreateWindowEx(0, szClassName, szAppTitle, WS_OVERLAPPEDWINDOW,
                        CW_USEDEFAULT, CW_USEDEFAULT, 280, 300, HWND_DESKTOP,
                        NULL, hThisInstance, NULL);

  hwndClear =
      CreateWindowEx(0, szButton, "*", WS_CHILD | WS_VISIBLE, 10, 10, 30, 30,
                     hwnd, (HMENU)BTN_CLEAR, hThisInstance, NULL);

  for (x = 0; x < 7; ++x) {
    for (y = 0; y < 7; ++y) {
      if (InCross(x, y)) {
        pegs[x][y] = CreateWindowEx(0, szButton, szEmpty, WS_CHILD | WS_VISIBLE,
                                    10 + y * 30, 10 + x * 30, 30, 30, hwnd,
                                    (HMENU)(x * 10 + y), hThisInstance, NULL);
      }
    }
  }

  Renew();

  ShowWindow(hwnd, SW_SHOWNORMAL);
  UpdateWindow(hwnd);

  while (GetMessage(&messages, NULL, 0, 0)) {
    TranslateMessage(&messages);
    DispatchMessage(&messages);
  }

  return messages.wParam;
}