(defparameter *PEG* #\x)
(defparameter *HOLE* #\-)
(defparameter *LINE* 12)
(defparameter *LF* (string #\linefeed))
(defparameter *DIRECTIONS* (list -1 (- *LINE*) 1 *LINE*))

(defvar *board* (concatenate 'string
    "..........." *LF*
    "..........." *LF*
    "....xxx...." *LF*
    "....xxx...." *LF*
    "..xxxxxxx.." *LF*
    "..xxx-xxx.." *LF*
    "..xxxxxxx.." *LF*
    "....xxx...." *LF*
    "....xxx...." *LF*
    "..........." *LF*
    "..........." *LF*
))

(defvar *noOfTests* 0)

(defun canMove (positio direction)
    (incf *noOfTests*)
    (and
        (char= (elt *board* positio) *PEG*)
        (char= (elt *board* (+ positio direction)) *PEG*)
        (char= (elt *board* (+ positio direction direction)) *HOLE*)))

(defun makeMove (positio direction)
    (setf (elt *board* positio) *HOLE*)
    (setf (elt *board* (+ positio direction)) *HOLE*)
    (setf (elt *board* (+ positio direction direction)) *PEG*))

(defun removeMove (positio direction)
    (setf (elt *board* positio) *PEG*)
    (setf (elt *board* (+ positio direction)) *PEG*)
    (setf (elt *board* (+ positio direction direction)) *HOLE*))

(defun solve ()
    (let ((pegsOnBoard 0) isSolved)
        (dotimes (positio (length *board*))
            (when (char= (elt *board* positio) *PEG*)
                (incf pegsOnBoard)
                (dolist (direction *DIRECTIONS*)
                    (when (canMove positio direction)
                        (makeMove positio direction)
                        (setf isSolved (solve))
                        (removeMove positio direction)
                        (when isSolved
                            (format t "~a~%" *board*)
                            (return-from solve 't))
                    ))
            ))
        (when (= pegsOnBoard 1)
            (format t "~a~%" *board*)
            t)))

(defun main ()
    (if (solve)
        (format t "Решение найдено.~%")
        (format t "Решение не найдено.~%"))
    (format t "Число проверок: ~d.~%" *noOfTests*))

(main)

(exit)